function [attrib_feature, TT] = attrib_filter_PR(image, attribute, threshold, dual)
%ATTRIB_FILTER filters a grayscale image by removing all objects for which
%a certain attribute of choice does not meet a predefined threshold
%criterion.
%
%   ATTRIB_FEATURE = ATTRIB_FILTER_PR(IMAGE, ATTRIBUTE, THRESHOLD, DUAL)
%   thresholds the input image on each of its gray values, resulting in a
%   set of binary images. For each of these binary images, objects are
%   considered, i.e. groups of pixels that are connected according to a
%   certain connectivity rule. In this function, 8-connectivity is used.
%   For each of the objects, an attribute is considered, for instance
%   'area', 'moment of inertia' or 'standard deviation'. In each of the
%   binary images, all objects are removed that do not satisfy a certain
%   predefined threshold. Finally, a filtered grayscale image is
%   reconstructed by assigning to each pixel the maximum gray value,
%   corresponding to the order of the binary images, for which the
%   criterion is still satisfied.
%
%   ATTRIB_FEATURE is a filtered grayscale image of the same size as the
%   input image.
%
%   IMAGE is the input grayscale image. Gray values should be discrete!
%   ATTRIBUTE is type of the attribute that is considered. Possible values
%   are 'area', 'mi' and 'sd, corresponding to 'area', 'moment of inertia'
%   and 'standard deviation', respectively.
%   THRESHOLD is the threshold criterion for the selected attribute.
%   DUAL (optional) selects between opening (default) and closing
%   (thickening or thinning), which are dual operations. 
%
%
%       Copyright notes
%       Author: Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%
%
%
%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~exist('dual','var')
    dual = 'open';
end
MGV=max(image(:));
switch dual
    case 'close'
        image = MGV - image;
    otherwise
end
[r,c] = size(image);
grayvalue = unique(image(:));
numofgrayvalues = length(grayvalue);
Ilayers = zeros(r,c,2);
TT=zeros(r,c,numofgrayvalues);
for i = 1:numofgrayvalues
    
    Igi = image;
    Igi(Igi < grayvalue(i)) = 0;
    Igi(Igi >= grayvalue(i)) = 1;
    cc = bwconncomp(Igi);
    BW2 = zeros(size(image));
    switch attribute
        case 'area'
            areacc = regionprops(cc,'Area');
            idx = find([areacc.Area] > threshold);
            if any(idx)
                BW2 = ismember(labelmatrix(cc), idx);
            end
            No_scales=3;
            
            for jj=1:No_scales
                BW21 = zeros(size(image));BW22 = zeros(size(image));
                Unconnected=getUnconnect(BW2, jj);
                FBW=BW2-Unconnected;%figure;imagesc(FBW)
                cc1 = bwconncomp(Unconnected);
                areacc1 = regionprops(cc1,'Area');
                idx1 = find([areacc1.Area] <= threshold);
                cc2 = bwconncomp(FBW);
                areacc2 = regionprops(cc2,'Area');
                idx2 = find([areacc2.Area] <= threshold);
                if any(idx1)
                    BW21 = ismember(labelmatrix(cc1), idx1);
                    BW2=BW2-BW21;
                end
                
                if any(idx2)
                    BW22 = ismember(labelmatrix(cc2), idx2);
                    BW2=BW2-BW22;
                end   
                
            end
            Ilayers(:,:,2) = BW2*grayvalue(i);
            TT(:,:,i)=BW2*grayvalue(i); 
                
        case 'mi'
            imagecc = regionprops(cc,'Image');
            len = length(imagecc);
            mi_reg = false(len,1);
            for k = 1:len
                mom_inertia = moments(imagecc(k).Image);
                if(mom_inertia > threshold)
                    mi_reg(k) = true;
                end
            end
            if any(mi_reg)
                BW2 = ismember(labelmatrix(cc), find(mi_reg));            
            end
            
            No_scales=3;
            
            for jj=1:No_scales
                BW21 = zeros(size(image));BW22 = zeros(size(image));
                Unconnected=getUnconnect(BW2, jj);
                FBW=BW2-Unconnected;%figure;imagesc(FBW)                
                cc1 = bwconncomp(Unconnected);
                cc2 = bwconncomp(FBW);
                imagecc1 = regionprops(cc1,'Image');
                len1 = length(imagecc1);
                mi_reg1 = false(len1,1);
                for k = 1:len1
                    mom_inertia1 = moments(imagecc1(k).Image);
                    if(mom_inertia1 <= threshold)
                        mi_reg1(k) = true;
                    end
                end
                imagecc2 = regionprops(cc2,'Image');
                len2 = length(imagecc2);
                mi_reg2 = false(len2,1);
                for k = 1:len2
                    mom_inertia2 = moments(imagecc2(k).Image);
                    if(mom_inertia2 <= threshold)
                        mi_reg2(k) = true;
                    end
                end
                if any(mi_reg1)
                    BW21 = ismember(labelmatrix(cc1), find(mi_reg1));
                    BW2=BW2-BW21;
                end
                if any(mi_reg2)
                    BW22 = ismember(labelmatrix(cc2), find(mi_reg2));
                    BW2=BW2-BW22;
                end            
            end
            
            Ilayers(:,:,2) = BW2*grayvalue(i);
        case 'sd'
            pixelidcc = regionprops(cc,'PixelIdxList');
            len = length(pixelidcc);
            sd_reg = false(len,1);
            for k = 1:len
                pixelsid = double(image(pixelidcc(k).PixelIdxList));
                stdarea = std(pixelsid);
                if(stdarea > threshold)
                    sd_reg(k) = true;
                end
            end
            if any(sd_reg)
                BW2 = ismember(labelmatrix(cc), find(sd_reg));
            end
            No_scales=3;
            
            for jj=1:No_scales
                BW21 = zeros(size(image));BW22 = zeros(size(image));
                Unconnected=getUnconnect(BW2, jj);
                FBW=BW2-Unconnected;%figure;imagesc(FBW) 
                cc1 = bwconncomp(Unconnected);
                cc2 = bwconncomp(FBW);
                pixelidcc1 = regionprops(cc1,'PixelIdxList');
                len1 = length(pixelidcc1);
                sd_reg1 = false(len1,1);
                for k = 1:len1
                    pixelsid1 = double(image(pixelidcc1(k).PixelIdxList));
                    stdarea1 = std(pixelsid1);
                    if(stdarea1 <= threshold)
                        sd_reg1(k) = true;
                    end
                end
                if any(sd_reg1)
                    BW21 = ismember(labelmatrix(cc1), find(sd_reg1));
                    BW2=BW2-BW21;
                end
                pixelidcc2 = regionprops(cc2,'PixelIdxList');
                len2 = length(pixelidcc2);
                sd_reg2 = false(len2,1);
                for k = 1:len2
                    pixelsid2 = double(image(pixelidcc2(k).PixelIdxList));
                    stdarea2 = std(pixelsid2);
                    if(stdarea2 <= threshold)
                        sd_reg2(k) = true;
                    end
                end
                if any(sd_reg2)
                    BW22 = ismember(labelmatrix(cc2), find(sd_reg2));
                    BW2=BW2-BW22;
                end            
            end
            Ilayers(:,:,2) = BW2*grayvalue(i);
    end
    Ilayers(:,:,1) = max(Ilayers,[],3);
    
end

attrib_feature = Ilayers(:,:,1);

switch dual
    case 'close'
        attrib_feature = MGV - attrib_feature;
    otherwise
end

    function MomentOfInertia = moments(I)
        % Calculates the moment of inertia of a binary image, the first Hu
        % invariant moment. This moment is analogous to the moment of
        % inertia around the image's centroid, where the pixels'
        % intensities are analogous to physical density.
        
        [numrows,numcols] = size(I);
        col = (1:numrows)';
        row = 1:numcols;
        X = repmat(col,[1 numcols]);
        Y = repmat(row,[numrows 1]);
        M00 = sum(sum(I));
        M10 = sum(sum(X.*I));
        M01 = sum(sum(Y.*I));
        M20 = sum(sum((X.^2).*I));
        M02 = sum(sum((Y.^2).*I));
        mu20 = M20-M10^2/M00;
        mu02 = M02-M01^2/M00;
        eita20 = mu20/(M00^2);
        eita02 = mu02/(M00^2);
        MomentOfInertia = eita20+eita02;
    end
end
