%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%
%
%       Copyright notes
%       Author: Rik Bellens and Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


classdef basestrel
    
    
    properties (SetAccess = protected)
        matlabse;
    end
    
    methods (Access = protected)
        
        function obj = basestrel(in)
            
            if isa(in,'strel')
                obj.matlabse = in;
            else
                error('...');
            end
        end
        
        function out = morphop(op,im,se,varargin)
            se = strel(se);
            out = feval(op,im,se,varargin{:});
        end
        
        function s = getboundarysize(se)
            s = (size(getnhood(strel(se)))-1)/2;
        end
        
        function fullim = generatefullim(im,se,b)
            s = getboundarysize(se);
            [h,w] = size(im);
            fullh = h+2*s(1);
            fullw = w+2*s(2);
            b = cast(b,class(im));
            fullim = zeros(fullh,fullw,class(im));
            fullim(:,1:s(2)) = b;
            fullim(:,w+s(2)+(1:s(2))) = b;
            fullim(1:s(1),:) = b;
            fullim(h+s(1)+(1:s(1)),:) = b;
            fullim( s(1) + (1:h), s(2) + (1:w) ) = im;
        end
        
    end
    
    methods (Static)
        function obj = CreateMatlabSE(type,varargin)
            obj = basestrel(strel(type,varargin{:}));
        end
        function obj = CreateBresenhamLineSE(l,a)
            obj = bresenhamlinestrel(l,a);
        end
        
        function obj = CreatePeriodicLineSE(P,V)
            obj = periodiclinestrel(P,V);
        end
        
        function obj = CreatePolygonSE(lambda,N)
            switch N
                case 8
                    V{1} = [0 1];
                    V{2} = [1 0];
                    V{3} = [1 1];
                    V{4} = [-1 1];
                case 12
                    V{1} = [0 1];
                    V{2} = [1 0];
                    V{3} = [1 2];
                    V{4} = [-1 2];
                    V{5} = [2 1];
                    V{6} = [-2 1];
                case 16
                    V{1} = [0 1];
                    V{2} = [1 0];
                    V{3} = [1 1];
                    V{4} = [-1 1];
                    V{5} = [1 2];
                    V{6} = [-1 2];
                    V{7} = [2 1];
                    V{8} = [-2 1];
                otherwise
                    error('...');
            end
            
            V = cell2mat(V');
            d = sqrt(sum(V.^2,2));

            l = (unique(d)*(1:lambda));
            l = sort(l(:));
            l = l(lambda);
            a = floor(l./d);
            
            for i=1:length(V)
                if a(i)>0, se(i) = periodiclinestrel(a(i),V(i,:));end
            end
            obj = sequencestrel(se);

        end
    end
    methods (Access = public)
        function r = getradius(se)
            n = getneighbors(strel(se));
            r = sqrt(max(sum(n.^2,2)));
        end
            
        function mse = strel(se)
            mse = strel('arbitrary',imdilate(1,se.matlabse,'full')==1);
        end
        
        
        function out = imdilate(im,se,varargin)
            out = morphop('imdilate',im,se,varargin{:});
        end

        function out = imerode(im,se,varargin)
            out = morphop('imerode',im,se,varargin{:});
        end
       
        function out = imopen(im,se,varargin)
            er = imerode(im,se,'full');
            out = imdilate(er,reflect(se));
            [h,w] = size(im);
            [fullh,fullw] = size(out);
            b = ([fullh,fullw]-[h,w])/2;
            out = out(b(1)+(1:h),b(2)+(1:w));
        end
        
        function out = imclose(im,se,varargin)
            di = imdilate(im,se,'full');
            out = imerode(di,reflect(se));
            [h,w] = size(im);
            [fullh,fullw] = size(out);
            b = ([fullh,fullw]-[h,w])/2;
            out = out(b(1)+(1:h),b(2)+(1:w));
        end
        
        function out = imgeodesicopen(im,se)
            er = imerode(im,se);
            out = imreconstruct(er,im);
        end
        
        function se = reflect(se)
            se.matlabse = reflect(se.matlabse);
        end
    end
    
end
